@@ -1,3 +1,4 @@
+"""Meta-programming tools to build declarative frameworks"""
 from i2.deco import (
     preprocess,
     postprocess,
@@ -12,4 +13,10 @@ from i2.deco import (
     input_output_decorator,
 )
 from i2.deco import wrap_class_methods_input_and_output
-from i2.signatures import Sig
+from i2.signatures import (
+    Sig,
+    Command,
+    extract_commands,
+    call_forgivingly,
+    call_somewhat_forgivingly,
+)
