@@ -2,30 +2,75 @@ import inspect
 from collections import defaultdict
 from collections.abc import Mapping
 from contextlib import suppress
-from functools import update_wrapper
-from functools import wraps
+from functools import wraps, partial, update_wrapper
 from inspect import Signature, signature, Parameter
 from itertools import chain
-from types import FunctionType
 from typing import Iterable
 
-from i2.signatures import ch_signature_to_all_pk, HasParams, VP, PK, Sig
+from i2.signatures import ch_func_to_all_pk, Sig, tuple_the_args
+
+# keep the imports below here because might be referenced (or take care of refs)
+from i2.signatures import ch_signature_to_all_pk, params_of, copy_func
 
 
-def copy_func(f):
-    """Copy a function (not sure it works with all types of callables)"""
-    g = FunctionType(
-        f.__code__,
-        f.__globals__,
-        name=f.__name__,
-        argdefs=f.__defaults__,
-        closure=f.__closure__,
-    )
-    g = update_wrapper(g, f)
-    g.__kwdefaults__ = f.__kwdefaults__
-    if hasattr(f, '__signature__'):
-        g.__signature__ = f.__signature__
-    return g
+def double_up_as_factory(decorator_func):
+    """Repurpose a decorator both as it's original form, and as a decorator factory.
+
+    That is, from a decorator that is defined do ``wrapped_func = decorator(func, **params)``,
+    make it also be able to do ``wrapped_func = decorator(**params)(func)``.
+
+    Note: You'll only be able to do this if all but the first argument are keyword-only,
+    and the first argument (the function to decorate) has a default of ``None`` (this is for your own good).
+    This is validated before making the "double up as factory" decorator.
+
+    >>> @double_up_as_factory
+    ... def decorator(func=None, *, multiplier=2):
+    ...     def _func(x):
+    ...         return func(x) * multiplier
+    ...     return _func
+    ...
+    >>> def foo(x):
+    ...     return x + 1
+    ...
+    >>> foo(2)
+    3
+    >>> wrapped_foo = decorator(foo, multiplier=10)
+    >>> wrapped_foo(2)
+    30
+    >>>
+    >>> multiply_by_3 = decorator(multiplier=3)
+    >>> wrapped_foo = multiply_by_3(foo)
+    >>> wrapped_foo(2)
+    9
+    >>>
+    >>> @decorator(multiplier=3)
+    ... def foo(x):
+    ...     return x + 1
+    ...
+    >>> foo(2)
+    9
+    """
+
+    def validate_decorator_func(decorator_func):
+        first_param, *other_params = signature(decorator_func).parameters.values()
+        assert (
+            first_param.default is None
+        ), f'First argument of the decorator function needs to default to None. Was {first_param.default}'
+        assert all(
+            p.kind == p.KEYWORD_ONLY for p in other_params
+        ), f'All arguments (besides the first) need to be keyword-only'
+        return True
+
+    validate_decorator_func(decorator_func)
+
+    @wraps(decorator_func)
+    def _double_up_as_factory(wrapped=None, **kwargs):
+        if wrapped is None:  # then we want a factory
+            return partial(decorator_func, **kwargs)
+        else:
+            return decorator_func(wrapped, **kwargs)
+
+    return _double_up_as_factory
 
 
 def transparently_wrapped(func):
@@ -36,58 +81,6 @@ def transparently_wrapped(func):
     return transparently_wrapped_func
 
 
-def params_of(obj: HasParams):
-    if isinstance(obj, Signature):
-        obj = list(obj.parameters.values())
-    elif isinstance(obj, Mapping):
-        obj = list(obj.values())
-    elif callable(obj):
-        obj = list(signature(obj).parameters.values())
-    assert all(
-        isinstance(p, Parameter) for p in obj
-    ), 'obj needs to be a Iterable[Parameter] at this point'
-    return obj  # as is
-
-
-def tuple_the_args(func):
-    """A decorator that will change a VAR_POSITIONAL (*args) argument to a tuple (args)
-    argument of the same name.
-    """
-    params = params_of(func)
-    is_vp = list(p.kind == VP for p in params)
-    if any(is_vp):
-        index_of_vp = is_vp.index(True)  # there's can be only one
-
-        @wraps(func)
-        def vpless_func(*args, **kwargs):
-            # extract the element of args that needs to be unraveled
-            a, _vp_args_, aa = (
-                args[:index_of_vp],
-                args[index_of_vp],
-                args[(index_of_vp + 1) :],
-            )
-            # call the original function with the unravelled args
-            return func(*a, *_vp_args_, *aa, **kwargs)
-
-        try:  # TODO: Avoid this try catch. Look in advance for default ordering
-            params[index_of_vp] = params[index_of_vp].replace(
-                kind=PK, default=()
-            )
-            vpless_func.__signature__ = Signature(
-                params, return_annotation=signature(func).return_annotation
-            )
-        except ValueError:
-            params[index_of_vp] = params[index_of_vp].replace(kind=PK)
-            vpless_func.__signature__ = Signature(
-                params, return_annotation=signature(func).return_annotation
-            )
-        return vpless_func
-    else:
-        return copy_func(
-            func
-        )  # don't change anything (or should we wrap anyway, to be consistent?)
-
-
 def mk_args_kwargs_merger(func):
     """
     Make a function that will return a dict containing all {argname: argval} pairs from a function's call.
@@ -116,9 +109,7 @@ def mk_args_kwargs_merger(func):
 
     def merge_args_and_kwargs(args, kwargs):
         if len(args) > 0:
-            return (
-                inspect.signature(func).bind_partial(*args, **kwargs).arguments
-            )
+            return inspect.signature(func).bind_partial(*args, **kwargs).arguments
         else:
             return kwargs
 
@@ -190,6 +181,7 @@ class MultiFunc:
      'mult': {'x': 2}}
 
     Oh, and you can actually see the signature of kwargs_for_func:
+
     >>> from inspect import signature
     >>> signature(mf1)
     <Signature (w, x: float, a, y=1, z: int = 1, b: float = 0.0)>
@@ -217,9 +209,7 @@ class MultiFunc:
     def __init__(self, funcs=()):
         self.funcs = ensure_iterable_of_callables(funcs)
         self.sigs = {func: Sig(func) for func in self.funcs}
-        self.normalized_funcs = {
-            func: self.normalize_func(func) for func in self.funcs
-        }
+        self.normalized_funcs = {func: self.normalize_func(func) for func in self.funcs}
         multi_func_sig = Sig.from_objs(*self.normalized_funcs.values())
         # TODO: Finish attempt to add **all_other_kwargs_ignored to the signature
         # multi_func_sig = (Sig.from_objs(
@@ -230,16 +220,14 @@ class MultiFunc:
 
     def kwargs_for_func(self, *args, **kwargs):
         return dict(
-            (func, self.sigs[func].source_kwargs(**kwargs))
-            for func in self.funcs
+            (func, self.sigs[func].source_kwargs(**kwargs)) for func in self.funcs
         )
 
     # TODO: Give it a signature (needs to be done in __init__)
     # TODO: Validation of inputs
     def __call__(self, *args, **kwargs):
         return dict(
-            (func, self.sigs[func].source_kwargs(**kwargs))
-            for func in self.funcs
+            (func, self.sigs[func].source_kwargs(**kwargs)) for func in self.funcs
         )
 
 
@@ -248,6 +236,7 @@ def assert_attrs(attrs):
     Asserts, at construction time, that the class contains a specific set of attributes
     :param attrs: An attribute name (string) or a list of attribute names whose existence needs to be enforced.
     :return: A class decorator that will enforce the existence of the attrs when an instance is made
+
     >>> @assert_attrs('foo')
     ... class A:
     ...     bar = 10
@@ -345,9 +334,7 @@ class OutputPostProcessingError(RuntimeError):
     ...
 
 
-def postprocess(
-    post, caught_post_errors=(Exception,), verbose_error_message=False
-):
+def postprocess(post, caught_post_errors=(Exception,), verbose_error_message=False):
     """Add some post-processing after a function
 
     :param post: The function to apply to the output
@@ -410,9 +397,7 @@ def postprocess(
             try:
                 return post(output)
             except caught_post_errors as e:
-                msg = (
-                    f'Error when postprocessing output with post func: {func}'
-                )
+                msg = f'Error when postprocessing output with post func: {func}'
                 if verbose_error_message:
                     msg += '\n' + f'  output={output}'
                     if (
@@ -424,10 +409,7 @@ def postprocess(
                             + '  which was obtained by func(*args, **kwargs) where:'
                         )
                         msg += (
-                            '\n'
-                            + f'    args: {args}'
-                            + '\n'
-                            + f'    kwargs: {kwargs}'
+                            '\n' + f'    args: {args}' + '\n' + f'    kwargs: {kwargs}'
                         )
                 msg += '\n' + f'Error is: {e}'
                 raise OutputPostProcessingError(msg)
@@ -438,8 +420,7 @@ def postprocess(
         ):  # intended to catch cases where wrapper doesn't have a signature
             wrapper_signature = signature(wrapper)
             sig = Signature(
-                wrapper_signature.parameters.values(),
-                return_annotation=return_annot,
+                wrapper_signature.parameters.values(), return_annotation=return_annot,
             )
             wrapper.__signature__ = sig
 
@@ -553,6 +534,7 @@ def transform_args(dflt_trans_func=None, /, **trans_func_for_arg):
     :param rootdir: rootdir to be used for all name arguments of target function
     :param name_arg: the position (int) or argument name of the argument containing the name
     :return: a decorator
+
     >>> # Example with a plain function
     >>> def f(a, b, c='default_c'):
     ...     return "a={a}, b={b}, c={c}".format(a=a, b=b, c=c)
@@ -626,9 +608,7 @@ def transform_args(dflt_trans_func=None, /, **trans_func_for_arg):
 
                 for argname, trans_func in trans_func_for_arg.items():
                     if argname in val_of_argname:
-                        val_of_argname[argname] = trans_func(
-                            val_of_argname[argname]
-                        )
+                        val_of_argname[argname] = trans_func(val_of_argname[argname])
                 # apply transform functions to argument values
                 return func(**val_of_argname)
 
@@ -752,9 +732,7 @@ def transform_class_method_input_and_output(
     wrapped_method = transform_args(**arg_trans)(getattr(cls, method))
     if method_output_trans is not None:
         setattr(
-            cls,
-            method,
-            wrap_method_output(method_output_trans)(wrapped_method),
+            cls, method, wrap_method_output(method_output_trans)(wrapped_method),
         )
     else:
         setattr(cls, method, wrapped_method)
@@ -1069,9 +1047,7 @@ def wrap_instance_methods(
                     class_name = obj.__name__
                 else:
                     class_name = str(obj)
-                raise ValueError(
-                    "{} has no '{}' method!".format(class_name, method)
-                )
+                raise ValueError("{} has no '{}' method!".format(class_name, method))
         return obj
 
     return obj_wrapper
@@ -1084,6 +1060,7 @@ def mk_method_trans_spec_from_methods_specs_dict(methods_specs_dict):
         keys are method names (either a single string, or a tuple of strings)
         values are the trans_spec dicts that should be associated to those methods
     :return: A dict in the method_trans_spec (input of wrap_class_method) format.
+
     >>> methods_specs_dict = {}
     >>> methods_specs_dict['foo'] = {'x': str, 'y': int}
     >>> methods_specs_dict[('foo', 'bar')] = {'z': list, 'method_output_trans': float}
@@ -1113,7 +1090,7 @@ WhatToLog = Callable[[Callable, Args, Kwargs], Any]
 
 
 def _special_str(x: Any, max_len=100) -> str:
-    """ A util function for _call_signature"""
+    """A util function for _call_signature"""
     if isinstance(x, str):
         return "'" + x + "'"
     else:
@@ -1232,6 +1209,8 @@ def mk_call_logger(
                 log_output and logger(log_output(out))
                 return out
 
+            _func._logged_with = logger
+
             return _func
 
     else:
@@ -1244,36 +1223,61 @@ def mk_call_logger(
                 log_output and logger(log_output(out))
                 return out
 
+            _func._logged_with = logger
+
             return _func
 
     return log_calls
 
 
-def ch_func_to_all_pk(func):
-    """Returns a copy of the function where all arguments are of the PK kind.
-    (PK: Positional_or_keyword)
-
-    :param func: A callable
-    :return:
-
-    >>> from py2http.decorators import signature, ch_func_to_all_pk
-    >>>
-    >>> def f(a, /, b, *, c=None, **kwargs): ...
-    ...
-    >>> print(signature(f))
-    (a, /, b, *, c=None, **kwargs)
-    >>> ff = ch_func_to_all_pk(f)
-    >>> print(signature(ff))
-    (a, b, c=None, **kwargs)
-    >>> def g(x, y=1, *args, **kwargs): ...
-    ...
-    >>> print(signature(g))
-    (x, y=1, *args, **kwargs)
-    >>> gg = ch_func_to_all_pk(g)
-    >>> print(signature(gg))
-    (x, y=1, args=(), **kwargs)
-    """
-    func = tuple_the_args(func)
-    sig = signature(func)
-    func.__signature__ = ch_signature_to_all_pk(sig)
+def get_callable_from_factory_if_no_arguments(func_or_factory_thereof: Callable):
+    """Will return the input itself if it's a callable with at least one argument.
+    If not, it will consider it to be a factory, call it to get the actual
+    callable object that the user presumably is seeking to get"""
+    assert callable(
+        func_or_factory_thereof
+    ), f'{func_or_factory_thereof} needs to be callable'
+    if len(Sig(func_or_factory_thereof)) == 0:
+        # if func_or_factory_thereof has no arguments, assume it's a factory
+        func = func_or_factory_thereof()
+        # and make sure that now we have arguments
+        if not isinstance(func, Callable) or not len(Sig(func)) > 0:
+            raise ValueError(
+                'Your func_or_factory_thereof had no arguments, so I assumed it '
+                "was a factory, called it, but that didn't produce a "
+                "callable with at least one argument. So I don't know what to do."
+            )
+    else:
+        func = func_or_factory_thereof
     return func
+
+
+## This one didn't actually handle position only correctly (just signature)
+# def old_ch_func_to_all_pk(func):
+#     """Returns a copy of the function where all arguments are of the PK kind.
+#     (PK: Positional_or_keyword)
+#
+#     :param func: A callable
+#     :return:
+#
+#     >>> from py2http.decorators import signature, ch_func_to_all_pk
+#     >>>
+#     >>> def f(a, /, b, *, c=None, **kwargs): ...
+#     ...
+#     >>> print(signature(f))
+#     (a, /, b, *, c=None, **kwargs)
+#     >>> ff = old_ch_func_to_all_pk(f)
+#     >>> print(signature(ff))
+#     (a, b, c=None, **kwargs)
+#     >>> def g(x, y=1, *args, **kwargs): ...
+#     ...
+#     >>> print(signature(g))
+#     (x, y=1, *args, **kwargs)
+#     >>> gg = old_ch_func_to_all_pk(g)
+#     >>> print(signature(gg))
+#     (x, y=1, args=(), **kwargs)
+#     """
+#     func = tuple_the_args(func)
+#     sig = signature(func)
+#     func.__signature__ = ch_signature_to_all_pk(sig)
+#     return func
