@@ -300,6 +300,7 @@ class StrKeyPath(KeyPathMap):
     """
     A KeyPathMap, but where the key paths are expressed as string with a separator.
     If sep = '.', then instead of using ('a', 'b', 'c') as a key, you can use 'a.b.c'.
+
     >>> d = {'a': {
     ...         'a': '2a',
     ...         'b': {'a': 'aba',
@@ -397,9 +398,7 @@ class StrKeyPath(KeyPathMap):
         yield from ((self._key_of_id(_id), v) for _id, v in super().items())
 
     def rollout(self, d):
-        target = self.__class__(
-            sep=self.sep, prefix=self.prefix, auto_node_writes=True
-        )
+        target = self.__class__(sep=self.sep, prefix=self.prefix, auto_node_writes=True)
         for k, v in d.items():
             target[k] = v
         return target.store
@@ -412,9 +411,7 @@ class KeyPathTrans:
     given to the method as input.
     """
 
-    def __init__(
-        self, sep: str = '.', node_type: type = dict, mk_new_node=None
-    ):
+    def __init__(self, sep: str = '.', node_type: type = dict, mk_new_node=None):
         """
 
         :param sep:
@@ -435,6 +432,7 @@ class KeyPathTrans:
         :param d: input store
         :param key_path_so_far: string to be prepended to all key paths (for use in recursion, not meant for direct use)
         :return: a (key_path, val) iterator
+
         >>> kp = KeyPathTrans()
         >>> input_dict = {
         ...     'a': {
@@ -506,6 +504,7 @@ class KeyPathTrans:
         :param key_path:
         :param val:
         :return:
+
         >>> kp = KeyPathTrans()
         >>> input_dict = {
         ...   "a": {
@@ -543,12 +542,7 @@ class KeyPathTrans:
                 self.setitem_recursive(d[first_key], key_path[1:], val)
 
     def extract_key_paths(
-        self,
-        d,
-        key_paths,
-        field_naming='full',
-        use_default=False,
-        default_val=None,
+        self, d, key_paths, field_naming='full', use_default=False, default_val=None,
     ):
         """
         getting with a key list or "."-separated string
@@ -557,6 +551,7 @@ class KeyPathTrans:
         :param field_naming: 'full' (default) will use key_path strings as is, leaf will only use the last dot item
             (i.e. this.is.a.key.path will result in "path" being used)
         :return:
+
         >>> kp = KeyPathTrans()
         >>> d = {
         ...     'a': {
